@@ -1,10 +1,10 @@
 /*
 
-  $Id: gsm-statemachine.c,v 1.53 2004/06/09 13:58:14 bozo Exp $
+  $Id: gsm-statemachine.c,v 1.62 2007/04/12 18:58:53 pkot Exp $
 
   G N O K I I
 
-  A Linux/Unix toolset and driver for Nokia mobile phones.
+  A Linux/Unix toolset and driver for the mobile phones.
 
   This file is part of gnokii.
 
@@ -22,7 +22,12 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 1999, 2000 Hugh Blemings & Pavel Jank ml.
+  Copyright (C) 1999-2000 Hugh Blemings & Pavel Jank ml.
+  Copyright (C) 2001      Chris Kemp, Manfred Jonsson
+  Copyright (C) 2001-2002 Pavel Machek, Ladis Michl
+  Copyright (C) 2001-2003 Pawel Kot
+  Copyright (C) 2002-2004 BORBELY Zoltan
+  Copyright (C) 2003      Osma Suominen
 
 */
 
@@ -43,8 +48,8 @@ gn_error sm_message_send(u16 messagesize, u8 messagetype, void *message, struct
 {
 	if (state->current_state != GN_SM_Startup) {
 #ifdef	DEBUG
-		dump("Message sent: ");
-		sm_message_dump(messagetype, message, messagesize);
+		dprintf("Message sent: ");
+		sm_message_dump(gn_log_debug, messagetype, message, messagesize);
 #endif
 		state->last_msg_size = messagesize;
 		state->last_msg_type = messagetype;
@@ -57,7 +62,7 @@ gn_error sm_message_send(u16 messagesize, u8 messagetype, void *message, struct
 	else return GN_ERR_NOTREADY;
 }
 
-API gn_state gn_sm_loop(int timeout, struct gn_statemachine *state)
+GNOKII_API gn_state gn_sm_loop(int timeout, struct gn_statemachine *state)
 {
 	struct timeval loop_timeout;
 	int i;
@@ -100,8 +105,8 @@ void sm_incoming_function(u8 messagetype, void *message, u16 messagesize, struct
 	int waitingfor = -1;
 
 #ifdef	DEBUG
-	dump("Message received: ");
-	sm_message_dump(messagetype, message, messagesize);
+	dprintf("Message received: ");
+	sm_message_dump(gn_log_debug, messagetype, message, messagesize);
 #endif
 	edata = (gn_data *)calloc(1, sizeof(gn_data));
 	data = edata;
@@ -129,8 +134,12 @@ void sm_incoming_function(u8 messagetype, void *message, u16 messagesize, struct
 		dprintf("Unsolicited frame, skipping...\n");
 		free(edata);
 		return;
-	} else if (res == GN_ERR_UNHANDLEDFRAME)
+	} else if (res == GN_ERR_UNHANDLEDFRAME) {
 		sm_unhandled_frame_dump(messagetype, message, messagesize, state);
+	} else if (res == GN_ERR_WAITING) {
+		free(edata);
+		return;
+	}
 	if (temp != 0) {
 		dprintf("Unknown Frame Type %02x\n", messagetype);
 		state->driver.default_function(messagetype, message, messagesize, state);
@@ -159,7 +168,8 @@ gn_error sm_error_get(unsigned char messagetype, struct gn_statemachine *state)
 	int c, d;
 	gn_error error = GN_ERR_NOTREADY;
 
-	if (state->current_state == GN_SM_ResponseReceived) {
+	switch (state->current_state) {
+	case GN_SM_ResponseReceived:
 		for (c = 0; c < state->received_number; c++)
 			if (state->waiting_for[c] == messagetype) {
 				error = state->response_error[c];
@@ -176,6 +186,13 @@ gn_error sm_error_get(unsigned char messagetype, struct gn_statemachine *state)
 			state->waiting_for_number = 0;
 			state->current_state = GN_SM_Initialised;
 		}
+		break;
+	/* Here we are fine! */
+	case GN_SM_Initialised:
+		error = GN_ERR_NONE;
+		break;
+	default:
+		break;
 	}
 
 	return error;
@@ -202,7 +219,8 @@ gn_error sm_wait_for(unsigned char messagetype, gn_data *data, struct gn_statema
 
 void sm_incoming_acknowledge(struct gn_statemachine *state)
 {
-	state->current_state = GN_SM_WaitingForResponse;
+	if (state->current_state == GN_SM_MessageSent)
+		state->current_state = GN_SM_WaitingForResponse;
 }
 
 
@@ -233,12 +251,19 @@ static gn_error __sm_block_timeout(int waitfor, int t, gn_data *data, struct gn_
 
 		if (s == GN_SM_WaitingForResponse || s == GN_SM_ResponseReceived) break;
 
-		dprintf("SM_Block Retry - %d\n", retry);
-		sm_reset(state);
-		sm_message_send(state->last_msg_size, state->last_msg_type, state->last_msg, state);
+		if (state->config.sm_retry) {
+			dprintf("SM_Block Retry - %d\n", retry);
+			sm_reset(state);
+			sm_message_send(state->last_msg_size, state->last_msg_type, state->last_msg, state);
+		} else {
+			/* If we don't want retry, we should exit the loop */
+			dprintf("SM_Block: exiting the retry loop\n");
+			break;
+		}
 	}
 
-	if (s == GN_SM_ResponseReceived) return sm_error_get(waitfor, state);
+	if (s == GN_SM_ResponseReceived)
+		return sm_error_get(waitfor, state);
 
 	timeout.tv_sec = t / 10;
 	timeout.tv_usec = (t % 10) * 100000;
@@ -248,7 +273,8 @@ static gn_error __sm_block_timeout(int waitfor, int t, gn_data *data, struct gn_
 		gettimeofday(&now, NULL);
 	} while (timercmp(&next, &now, >) && (s != GN_SM_ResponseReceived));
 
-	if (s == GN_SM_ResponseReceived) return sm_error_get(waitfor, state);
+	if (s == GN_SM_ResponseReceived)
+		return sm_error_get(waitfor, state);
 
 	sm_reset(state);
 
@@ -320,7 +346,7 @@ gn_error sm_block_ack(struct gn_statemachine *state)
 }
 
 /* Just to do things neatly */
-API gn_error gn_sm_functions(gn_operation op, gn_data *data, struct gn_statemachine *sm)
+GNOKII_API gn_error gn_sm_functions(gn_operation op, gn_data *data, struct gn_statemachine *sm)
 {
 	if (!sm->driver.functions) {
 		dprintf("Sorry, phone has not yet been converted to new style. Phone.Functions == NULL!\n");
@@ -330,37 +356,37 @@ API gn_error gn_sm_functions(gn_operation op, gn_data *data, struct gn_statemach
 }
 
 /* Dumps a message */
-void sm_message_dump(int messagetype, unsigned char *message, int messagesize)
+void sm_message_dump(gn_log_func_t lfunc, int messagetype, unsigned char *message, int messagesize)
 {
 	int i;
 	char buf[17];
 
 	buf[16] = 0;
 
-	dump("0x%02x / 0x%04x", messagetype, messagesize);
+	lfunc("0x%02x / 0x%04x", messagetype, messagesize);
 
 	for (i = 0; i < messagesize; i++) {
 		if (i % 16 == 0) {
-			if (i != 0) dump("| %s", buf);
-			dump("\n");
+			if (i != 0) lfunc("| %s", buf);
+			lfunc("\n");
 			memset(buf, ' ', 16);
 		}
-		dump("%02x ", message[i]);
+		lfunc("%02x ", message[i]);
 		if (isprint(message[i])) buf[i % 16] = message[i];
 	}
 
-	if (i != 0) dump("%*s| %s", i % 16 ? 3 * (16 - i % 16) : 0, "", buf);
-	dump("\n");
+	if (i != 0) lfunc("%*s| %s", i % 16 ? 3 * (16 - i % 16) : 0, "", buf);
+	lfunc("\n");
 }
 
 /* Prints a warning message about unhandled frames */
 void sm_unhandled_frame_dump(int messagetype, unsigned char *message, int messagesize, struct gn_statemachine *state)
 {
 	dump(_("UNHANDLED FRAME RECEIVED\nrequest: "));
-	sm_message_dump(state->last_msg_type, state->last_msg, state->last_msg_size);
+	sm_message_dump(gn_elog_write, state->last_msg_type, state->last_msg, state->last_msg_size);
 
 	dump(_("reply: "));
-	sm_message_dump(messagetype, message, messagesize);
+	sm_message_dump(gn_elog_write, messagetype, message, messagesize);
 
 	dump(_("Please read Docs/Bugs and send a bug report!\n"));
 }
