@@ -1,10 +1,10 @@
 /*
 
-  $Id: fbus.c,v 1.57 2004/04/29 22:13:12 pkot Exp $
+  $Id: fbus.c,v 1.64 2007/02/25 15:05:54 dforsi Exp $
 
   G N O K I I
 
-  A Linux/Unix toolset and driver for Nokia mobile phones.
+  A Linux/Unix toolset and driver for the mobile phones.
 
   This file is part of gnokii.
 
@@ -22,8 +22,13 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 2000 Hugh Blemings & Pavel Jank ml.
-  Copyright (C) 2000 Chris Kemp
+  Copyright (C) 2000      Hugh Blemings & Pavel Jank ml.
+  Copyright (C) 2000-2001 Chris Kemp
+  Copyright (C) 2001-2006 Pawel Kot
+  Copyright (C) 2001      Manfred Jonsson, Pavel Machek
+  Copyright (C) 2001-2002 Ladis Michl
+  Copyright (C) 2002-2004 BORBELY Zoltan
+  Copyright (C) 2003      Marcel Holtmann
 
   This file provides an API for accessing functions via fbus.
   See README for more details on supported mobile phones.
@@ -54,7 +59,7 @@ static int fbus_tx_send_ack(u8 message_type, u8 message_seq, struct gn_statemach
 
 /* FIXME - win32 stuff! */
 
-#define FBUSINST(s) ((fbus_link *)((s)->link.link_instance))
+#define FBUSINST(s) (*((fbus_link **)(&(s)->link.link_instance)))
 
 #define	IR_MODE(s) ((s)->config.connection_type == GN_CT_Infrared || (s)->config.connection_type == GN_CT_Tekram)
 
@@ -87,12 +92,36 @@ static bool fbus_serial_open(bool dlr3, struct gn_statemachine *state)
 	return true;
 }
 
+static int send_command(char *cmd, int len, struct gn_statemachine *state)
+{
+	struct timeval timeout;
+	unsigned char buffer[255];
+	int res;
+
+	/* Communication with the phone looks strange here. I am unable to
+	 * read the whole answer from the port with DKU-5 cable and
+	 * Nokia 6020. The following code seems to work reliably.
+	 */
+	device_write(cmd, len, state);
+	/* Experimental timeout */
+	timeout.tv_sec	= 0;
+	timeout.tv_usec	= 500000;
+        
+	res = device_select(&timeout, state);
+	/* Read from the port only when select succeeds */
+	while (res > 0) {
+		/* Avoid 'device temporarily unavailable' error */
+		usleep(50);
+		res = device_read(buffer, 255, state);
+	}
+	return res;
+}
+
 static bool at2fbus_serial_open(struct gn_statemachine *state, gn_connection_type type)
 {
 	unsigned char init_char = 0x55;
 	unsigned char end_init_char = 0xc1;
 	int count, res;
-	unsigned char buffer[255];
 
 	if (!state)
 		return false;
@@ -104,20 +133,17 @@ static bool at2fbus_serial_open(struct gn_statemachine *state, gn_connection_typ
 	}
  
 	device_setdtrrts(0, 0, state);
-	sleep(1);
+	usleep(1000000);
 	device_setdtrrts(1, 1, state);
+	usleep(1000000);
 	device_changespeed(19200, state);
-	sleep(1);
-	device_write("AT\r", 3, state);
-	sleep(1);
-	res = device_read(buffer, 255, state);
-	device_write("AT&F\r", 5, state);
-	usleep(100000);
-	res = device_read(buffer, 255, state);
-	device_write("AT*NOKIAFBUS\r", 13, state);
-	usleep(100000);
-	res = device_read(buffer, 255, state);
- 
+	dprintf("Switching to FBUS mode\n");
+	/* Here we can be either switched to FBUS or not. Assume 0.5 second
+	 * timeout for the answer.
+	 */
+	res = send_command("AT\r\n", 4, state);
+	res = send_command("AT&F\r\n", 6, state);
+	res = send_command("AT*NOKIAFBUS\r\n", 14, state);
 	device_changespeed(115200, state);
 
 	if (type != GN_CT_Bluetooth && type != GN_CT_TCP) { 
@@ -252,9 +278,12 @@ static void fbus_rx_statemachine(unsigned char rx_byte, struct gn_statemachine *
 		/* Then all sorts of horrible things happen because the packet length etc is wrong... */
 		/* Therefore we test here for a destination of 0x0c and return to the top if it is not */
 
-		if (rx_byte != 0x0c) {
+		if (rx_byte == 0x00) {
+			i->state = FBUS_RX_EchoSource;
+
+		} else if (rx_byte != 0x0c) {
 			i->state = FBUS_RX_Sync;
-			dprintf("The fbus stream is out of sync - expected 0x0c, got %2x\n", rx_byte);
+			dprintf("The fbus stream is out of sync - expected 0x0c, got 0x%02x\n", rx_byte);
 		}
 
 		break;
@@ -268,7 +297,7 @@ static void fbus_rx_statemachine(unsigned char rx_byte, struct gn_statemachine *
 
 		if (rx_byte != 0x00) {
 			i->state = FBUS_RX_Sync;
-			dprintf("The fbus stream is out of sync - expected 0x00, got %2x\n",rx_byte);
+			dprintf("The fbus stream is out of sync - expected 0x00, got 0x%02x\n", rx_byte);
 		}
 
 		break;
@@ -298,7 +327,7 @@ static void fbus_rx_statemachine(unsigned char rx_byte, struct gn_statemachine *
 	case FBUS_RX_GetMessage:
 
 		if (i->buffer_count >= FBUS_FRAME_MAX_LENGTH) {
-			dprintf("FBUS: Message buffer overun - resetting\n");
+			dprintf("FBUS: Message buffer overrun - resetting\n");
 			i->state = FBUS_RX_Sync;
 			break;
 		}
@@ -326,6 +355,8 @@ static void fbus_rx_statemachine(unsigned char rx_byte, struct gn_statemachine *
 							     i->frame_length - 2, state);
 				} else {	/* Normal message type */
 
+					sm_incoming_acknowledge(state);
+
 					/* Add data to the relevant Message buffer */
 					/* having checked the sequence number */
 
@@ -347,7 +378,7 @@ static void fbus_rx_statemachine(unsigned char rx_byte, struct gn_statemachine *
 							m->message_buffer = NULL;
 						}
 						m->malloced = frm_num * m->message_length;
-						m->message_buffer = (char *) malloc(m->malloced);
+						m->message_buffer = (unsigned char *)malloc(m->malloced);
 
 					} else if (m->frames_to_go != frm_num) {
 						dprintf("Missed a frame in a multiframe message.\n");
@@ -356,7 +387,7 @@ static void fbus_rx_statemachine(unsigned char rx_byte, struct gn_statemachine *
 
 					if (m->malloced < m->message_length + i->frame_length) {
 						m->malloced = m->message_length + i->frame_length;
-						m->message_buffer = (char *) realloc(m->message_buffer, m->malloced);
+						m->message_buffer = (unsigned char *)realloc(m->message_buffer, m->malloced);
 					}
 
 					memcpy(m->message_buffer + m->message_length, i->message_buffer,
@@ -386,6 +417,56 @@ static void fbus_rx_statemachine(unsigned char rx_byte, struct gn_statemachine *
 			}
 		}
 		break;
+
+	case FBUS_RX_EchoSource:
+
+		i->message_source = rx_byte;
+		i->state = FBUS_RX_EchoType;
+
+		if (rx_byte != 0x0c) {
+			i->state = FBUS_RX_Sync;
+			dprintf("The fbus stream is out of sync - expected 0x0c, got 0x%02x\n", rx_byte);
+		}
+
+		break;
+
+	case FBUS_RX_EchoType:
+
+		i->message_type = rx_byte;
+		i->state = FBUS_RX_EchoLength1;
+
+		break;
+
+	case FBUS_RX_EchoLength1:
+
+		i->state = FBUS_RX_EchoLength2;
+
+		break;
+
+	case FBUS_RX_EchoLength2:
+
+		i->frame_length = rx_byte;
+		i->state = FBUS_RX_EchoMessage;
+		i->buffer_count = 0;
+
+		break;
+
+	case FBUS_RX_EchoMessage:
+
+		if (i->buffer_count >= FBUS_FRAME_MAX_LENGTH) {
+			dprintf("FBUS: Message buffer overrun - resetting\n");
+			i->state = FBUS_RX_Sync;
+			break;
+		}
+
+		i->buffer_count++;
+
+		if (i->buffer_count == i->frame_length + (i->frame_length % 2) + 2) {
+			i->state = FBUS_RX_Sync;
+			dprintf("[Echo cancelled]\n");
+		}
+
+		break;
 	}
 }
 
@@ -548,7 +629,7 @@ static int fbus_tx_send_ack(u8 message_type, u8 message_seq, struct gn_statemach
 
 
 /* Initialise variables and start the link */
-/* state is only passed around to allow for muliple state machines (one day...) */
+/* state is only passed around to allow for multiple state machines (one day...) */
 
 gn_error fbus_initialise(int attempt, struct gn_statemachine *state)
 {
