@@ -1,6 +1,6 @@
 /*
 
-  $Id: gnokii.c,v 1.469 2006/08/09 19:52:19 pkot Exp $
+  $Id: gnokii.c,v 1.480 2007/05/26 13:48:54 dforsi Exp $
 
   G N O K I I
 
@@ -200,7 +200,7 @@ static void short_version(void)
 	fprintf(stderr, _("GNOKII Version %s\n"), VERSION);
 }
 
-/* This function shows the copyright and some informations usefull for
+/* This function shows the copyright and some informations useful for
    debugging. */
 static void version(void)
 {
@@ -246,6 +246,45 @@ static int usage(FILE *f, int retval)
 	exit(retval);
 }
 
+static void gnokii_error_logger(const char *fmt, va_list ap)
+{
+	if (logfile) {
+		vfprintf(logfile, fmt, ap);
+		fflush(logfile);
+	}
+}
+
+static int install_log_handler(void)
+{
+	char logname[256];
+	char *home;
+#ifdef WIN32
+	char *file = "gnokii-errors";
+#else
+	char *file = ".gnokii-errors";
+#endif
+
+	if ((home = getenv("HOME")) == NULL) {
+#ifdef WIN32
+		home = ".";
+#else
+		fprintf(stderr, _("ERROR: Can't find HOME environment variable!\n"));
+		return -1;
+#endif
+	}
+
+	snprintf(logname, sizeof(logname), "%s/%s", home, file);
+
+	if ((logfile = fopen(logname, "a")) == NULL) {
+		fprintf(stderr, _("Cannot open logfile %s\n"), logname);
+		return -1;
+	}
+
+	gn_elog_handler = gnokii_error_logger;
+
+	return 0;
+}
+
 /* businit is the generic function which waits for the FBUS link. The limit
    is 10 seconds. After 10 seconds we quit. */
 
@@ -276,6 +315,10 @@ static void businit(void)
 	atexit(busterminate);
 	/* signal(SIGINT, bussignal); */
 
+	if (install_log_handler()) {
+		fprintf(stderr, _("WARNING: cannot open logfile, logs will be directed to stderr\n"));
+	}
+
 	if ((err = gn_lib_phone_open(state)) != GN_ERR_NONE) {
 		fprintf(stderr, "%s\n", gn_error_print(err));
 		exit(2);
@@ -283,7 +326,7 @@ static void businit(void)
 	data = &state->sm_data;
 }
 
-/* This function checks that the argument count for a given options is withing
+/* This function checks that the argument count for a given options is within
    an allowed range. */
 static int checkargs(int opt, struct gnokii_arg_len gals[], int argc, int has_arg)
 {
@@ -326,52 +369,14 @@ static int checkargs(int opt, struct gnokii_arg_len gals[], int argc, int has_ar
 /* This is a "convenience" function to allow quick test of new API stuff which
    doesn't warrant a "proper" command line function. */
 #ifndef WIN32
-static int foogle(int argc, char *argv[], gn_data *data, struct gn_statemachine *state)
-{	
+static int foogle(int argc, char *argv[])
+{
 	/* Fill in what you would like to test here... */
+	/* remember to call businit(); if you need to communicate with your phone */
 	return 0;
 }
 #endif
 
-static void gnokii_error_logger(const char *fmt, va_list ap)
-{
-	if (logfile) {
-		vfprintf(logfile, fmt, ap);
-		fflush(logfile);
-	}
-}
-
-static int install_log_handler(void)
-{
-	char logname[256];
-	char *home;
-#ifdef WIN32
-	char *file = "gnokii-errors";
-#else
-	char *file = ".gnokii-errors";
-#endif
-
-	if ((home = getenv("HOME")) == NULL) {
-#ifdef WIN32
-		home = ".";
-#else
-		fprintf(stderr, _("ERROR: Can't find HOME environment variable!\n"));
-		return -1;
-#endif
-	}
-
-	snprintf(logname, sizeof(logname), "%s/%s", home, file);
-
-	if ((logfile = fopen(logname, "a")) == NULL) {
-		fprintf(stderr, _("Cannot open logfile %s\n"), logname);
-		return -1;
-	}
-
-	gn_elog_handler = gnokii_error_logger;
-
-	return 0;
-}
-
 static int parse_options(int argc, char *argv[])
 {
 	int c, rc = -1;
@@ -379,7 +384,7 @@ static int parse_options(int argc, char *argv[])
 
 	/* Every option should be in this array. */
 	static struct option long_options[] = {
-		/* FIXME: these comments are nice, but they would be more usefull as docs for the user */
+		/* FIXME: these comments are nice, but they would be more useful as docs for the user */
 		/* Display usage. */
 		{ "help",               no_argument,       NULL, OPT_HELP },
 
@@ -396,6 +401,9 @@ static int parse_options(int argc, char *argv[])
 
 #ifdef SECURITY
 
+		/* Get Security Code */
+		{ "getsecuritycode",    no_argument,   	   NULL, OPT_GETSECURITYCODE },
+
 		/* Enter Security Code mode */
 		{ "entersecuritycode",  required_argument, NULL, OPT_ENTERSECURITYCODE },
 
@@ -531,7 +539,7 @@ static int parse_options(int argc, char *argv[])
 		{ "reset",              required_argument, NULL, OPT_RESET },
 
 		/* Set logo */
-		{ "setlogo",            optional_argument, NULL, OPT_SETLOGO },
+		{ "setlogo",            required_argument, NULL, OPT_SETLOGO },
 
 		/* Get logo */
 		{ "getlogo",            required_argument, NULL, OPT_GETLOGO },
@@ -545,10 +553,10 @@ static int parse_options(int argc, char *argv[])
 		/* Set profile */
 		{ "setprofile",         no_argument,       NULL, OPT_SETPROFILE },
 
-		/* Gets the active profile */
+		/* Get the active profile */
 		{ "getactiveprofile",   no_argument,       NULL, OPT_GETACTIVEPROFILE },
 
-		/* Sets the active profile */
+		/* Set the active profile */
 		{ "setactiveprofile",   required_argument, NULL, OPT_SETACTIVEPROFILE },
 
 		/* Show texts from phone's display */
@@ -561,7 +569,7 @@ static int parse_options(int argc, char *argv[])
 		{ "enterchar",          no_argument,       NULL, OPT_ENTERCHAR },
 
 		/* Divert calls */
-		{ "divert",		required_argument, NULL, OPT_DIVERT },
+		{ "divert",		no_argument,       NULL, OPT_DIVERT },
 
 		/* SMS reader */
 		{ "smsreader",          no_argument,       NULL, OPT_SMSREADER },
@@ -569,9 +577,6 @@ static int parse_options(int argc, char *argv[])
 		/* For development purposes: insert you function calls here */
 		{ "foogle",             no_argument,       NULL, OPT_FOOGLE },
 
-		/* Get Security Code */
-		{ "getsecuritycode",    no_argument,   	   NULL, OPT_GETSECURITYCODE },
-
 		/* Get WAP bookmark */
 		{ "getwapbookmark",     required_argument, NULL, OPT_GETWAPBOOKMARK },
 
@@ -598,24 +603,34 @@ static int parse_options(int argc, char *argv[])
 
 		/* Get (sim)lock info */
 		{ "getlocksinfo",       no_argument,       NULL, OPT_GETLOCKSINFO },
+
                 /* Get file list */
 		{ "getfilelist",        required_argument, NULL, OPT_GETFILELIST },
+
                 /* Get file details by id */
 		{ "getfiledetailsbyid", optional_argument, NULL, OPT_GETFILEDETAILSBYID },
+
                 /* Get file id */
 		{ "getfileid",          required_argument, NULL, OPT_GETFILEID },
+
                 /* Get file */
 		{ "getfile",            required_argument, NULL, OPT_GETFILE },
+
                 /* Get file by id */
 		{ "getfilebyid",        required_argument, NULL, OPT_GETFILEBYID },
+
 		/* Get all files */
 		{ "getallfiles",        required_argument, NULL, OPT_GETALLFILES },
+
                 /* Put a file */
 		{ "putfile",            required_argument, NULL, OPT_PUTFILE },
+
 		/* Delete a file */
 		{ "deletefile",         required_argument, NULL, OPT_DELETEFILE },
+
 		/* Delete a file by id */
 		{ "deletefilebyid",     required_argument, NULL, OPT_DELETEFILEBYID },
+
 		{ 0, 0, 0, 0 },
 	};
 
@@ -634,9 +649,9 @@ static int parse_options(int argc, char *argv[])
 		{ OPT_ANSWERCALL,        1, 1, 0 },
 		{ OPT_HANGUP,            1, 1, 0 },
 		{ OPT_GETTODO,           1, 3, 0 },
-		{ OPT_WRITETODO,         2, 2, 0 },
+		{ OPT_WRITETODO,         2, 3, 0 },
 		{ OPT_GETCALENDARNOTE,   1, 3, 0 },
-		{ OPT_WRITECALENDARNOTE, 2, 2, 0 },
+		{ OPT_WRITECALENDARNOTE, 2, 3, 0 },
 		{ OPT_DELCALENDARNOTE,   1, 2, 0 },
 		{ OPT_GETPHONEBOOK,      2, 4, 0 },
 		{ OPT_WRITEPHONEBOOK,    0, 10, 0 },
@@ -740,15 +755,16 @@ static int parse_options(int argc, char *argv[])
 		configmodel = optarg;
 		return parse_options(argc, argv);
 	case OPT_VIEWLOGO:
-		rc = viewlogo(optarg);
-		break;
+		return viewlogo(optarg);
 	case OPT_LISTNETWORKS:
 		list_gsm_networks();
-		break;
+		return GN_ERR_NONE;
+	case OPT_RINGTONECONVERT:
+		return ringtoneconvert(argc, argv);
 	}
 
 	/* Initialise the code for the GSM interface. */
-	if (c != OPT_VIEWLOGO && c != OPT_FOOGLE && c != OPT_LISTNETWORKS && c != OPT_RINGTONECONVERT)
+	if (c != OPT_FOOGLE)
 		businit();
 
 	switch (c) {
@@ -814,18 +830,18 @@ static int parse_options(int argc, char *argv[])
 		rc = getcalendarnote(argc, argv, data, state);
 		break;
 	case OPT_WRITECALENDARNOTE:
-		rc = writecalendarnote(argv, data, state);
+		rc = writecalendarnote(argc, argv, data, state);
 		break;
 	case OPT_DELCALENDARNOTE:
 		rc = deletecalendarnote(argc, argv, data, state);
 		break;
 
-	/* TODO options */
+	/* ToDo options */
 	case OPT_GETTODO:
 		rc = gettodo(argc, argv, data, state);
 		break;
 	case OPT_WRITETODO:
-		rc = writetodo(argv, data, state);
+		rc = writetodo(argc, argv, data, state);
 		break;
 	case OPT_DELETEALLTODOS:
 		rc = deletealltodos(data, state);
@@ -929,9 +945,6 @@ static int parse_options(int argc, char *argv[])
 	case OPT_PLAYRINGTONE:
 		rc = playringtone(argc, argv, data, state);
 		break;
-	case OPT_RINGTONECONVERT:
-		rc = ringtoneconvert(argc, argv, data, state);
-		break;
 	case OPT_GETRINGTONELIST:
 		rc = getringtonelist(data, state);
 		break;
@@ -946,10 +959,10 @@ static int parse_options(int argc, char *argv[])
 	case OPT_GETLOCKSINFO:
 		rc = getlocksinfo(data, state);
 		break;
+#ifdef SECURITY
 	case OPT_GETSECURITYCODE:
 		rc = getsecuritycode(data, state);
 		break;
-#ifdef SECURITY
 	case OPT_ENTERSECURITYCODE:
 		rc = entersecuritycode(optarg, data, state);
 		break;
@@ -1005,7 +1018,7 @@ static int parse_options(int argc, char *argv[])
 		break;
 #ifndef WIN32
 	case OPT_FOOGLE:
-		rc = foogle(argc, argv, data, state);
+		rc = foogle(argc, argv);
 		break;
 #endif
 	default:
@@ -1028,10 +1041,6 @@ int main(int argc, char *argv[])
 	setlocale(LC_ALL, "");
 #endif
 
-	if (install_log_handler()) {
-		fprintf(stderr, _("WARNING: cannot open logfile, logs will be directed to stderr\n"));
-	}
-
 	opterr = 0;
 
 	/* Introduce yourself */
