@@ -1,6 +1,6 @@
 /*
 
-  $Id: common.h,v 1.152 2007/05/04 19:41:34 pkot Exp $
+  $Id: common.h,v 1.154 2007/07/09 18:29:00 pkot Exp $
 
   G N O K I I
 
@@ -102,6 +102,7 @@ typedef enum {
 	GN_MT_F18,
 	GN_MT_F19,
 	GN_MT_F20, /* 20th CUSTOM FOLDER */
+	GN_MT_LAST = GN_MT_F20,
 	GN_MT_XX = 0xff	/* Error code for unknown memory type (returned by fbus-xxxx functions). */
 } gn_memory_type;
 
@@ -377,6 +378,7 @@ typedef struct {
 	char phone_number[GN_CALNOTE_NUMBER_MAX_LENGTH]; /* For Call only: the phone number */
 	char mlocation[GN_CALNOTE_MAX_LENGTH];           /* For Meeting only: the location field */
 	gn_calnote_recurrence recurrence;                /* Recurrence of the note */
+	int occurrences;                                 /* Number of recurrence events; 0 for infinity */
 } gn_calnote;
 
 /* List of Calendar Notes in phone */
