@@ -1,6 +1,6 @@
 /*
 
-  $Id: atgen.h,v 1.23 2007/04/14 19:36:35 pkot Exp $
+  $Id: atgen.h,v 1.28 2007/07/09 17:09:56 pkot Exp $
 
   G N O K I I
 
@@ -45,6 +45,8 @@ typedef enum {
 	GN_OP_AT_Prompt,
 	GN_OP_AT_GetMemoryRange,
 	GN_OP_AT_Ring,
+	GN_OP_AT_IncomingSMS,
+	GN_OP_AT_GetSMSMemorySize,
 	GN_OP_AT_Max	/* don't append anything after this entry */
 } at_operation;
 
@@ -72,6 +74,7 @@ typedef struct {
 	int if_pos;
 	int no_smsc;
 
+	/* CBPS (phonebook related) */
 	gn_memory_type memorytype;
 	int memoryoffset;
 	int memorysize;
@@ -80,7 +83,27 @@ typedef struct {
 	at_charset defaultcharset;
 	at_charset charset;
 
-	void (*call_notification)(gn_call_status call_status, gn_call_info *call_info, struct gn_statemachine *state);
+	/* CPMS (sms related) */
+	int smmemorysize;
+	int mememorysize;
+
+	/* For call notifications via AT+CLIP */
+	int clip_supported;
+	gn_call_type last_call_type;
+	gn_call_status last_call_status;
+
+	/* For AT+CPAS */
+	gn_call_status prev_state;
+
+	/* callbacks */
+	void (*on_cell_broadcast)(gn_cb_message *msg, struct gn_statemachine *state, void *callback_data);
+	void (*call_notification)(gn_call_status call_status, gn_call_info *call_info, struct gn_statemachine *state, void *callback_data);
+	gn_error (*on_sms)(gn_sms *message, struct gn_statemachine *state, void *callback_data);
+
+	/* callback local data */
+	void *cb_callback_data;	/* to be passed as callback_data to on_cell_broadcast */
+	void *call_callback_data;	/* to be passed as callback_data to call_notification */
+	void *sms_callback_data;	/* to be passed as callback_data to on_sms */
 } at_driver_instance;
 
 #define AT_DRVINST(s) (*((at_driver_instance **)(&(s)->driver.driver_instance)))
