@@ -1,6 +1,6 @@
 /*
 
-  $Id: xgnokii_netmon.c,v 1.17 2006/08/15 10:57:20 dforsi Exp $
+  $Id: xgnokii_netmon.c,v 1.18 2007/05/30 21:49:40 dforsi Exp $
 
   X G N O K I I
 
@@ -262,14 +262,14 @@ static GtkItemFactoryEntry menu_items[] = {
 
 static void InitMainMenu(void)
 {
-	menu_items[0].path = g_strdup(_("/_File"));
-	menu_items[1].path = g_strdup(_("/File/_Close"));
-	menu_items[2].path = g_strdup(_("/_Tools"));
-	menu_items[3].path = g_strdup(_("/Tools/Net monitor o_n"));
-	menu_items[4].path = g_strdup(_("/Tools/Net monitor o_ff"));
-	menu_items[5].path = g_strdup(_("/_Help"));
-	menu_items[6].path = g_strdup(_("/Help/_Help"));
-	menu_items[7].path = g_strdup(_("/Help/_About"));
+	menu_items[0].path = _("/_File");
+	menu_items[1].path = _("/File/_Close");
+	menu_items[2].path = _("/_Tools");
+	menu_items[3].path = _("/Tools/Net monitor o_n");
+	menu_items[4].path = _("/Tools/Net monitor o_ff");
+	menu_items[5].path = _("/_Help");
+	menu_items[6].path = _("/Help/_Help");
+	menu_items[7].path = _("/Help/_About");
 }
 
 
